@@ -15,15 +15,15 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#define ARROW_VERSION_MAJOR 3
+#define ARROW_VERSION_MAJOR 5
 #define ARROW_VERSION_MINOR 0
 #define ARROW_VERSION_PATCH 0
 #define ARROW_VERSION ((ARROW_VERSION_MAJOR * 1000) + ARROW_VERSION_MINOR) * 1000 + ARROW_VERSION_PATCH
 
-#define ARROW_VERSION_STRING "3.0.0"
+#define ARROW_VERSION_STRING "5.0.0"
 
-#define ARROW_SO_VERSION "300"
-#define ARROW_FULL_SO_VERSION "300.0.0"
+#define ARROW_SO_VERSION "500"
+#define ARROW_FULL_SO_VERSION "500.0.0"
 
 #define ARROW_CXX_COMPILER_ID "GNU"
 #define ARROW_CXX_COMPILER_VERSION "10.2.0"
@@ -34,6 +34,14 @@
 
 #define ARROW_PACKAGE_KIND ""
 
+#define ARROW_COMPUTE
+#define ARROW_CSV
+#define ARROW_DATASET
+#define ARROW_FILESYSTEM
+#define ARROW_FLIGHT
+#define ARROW_IPC
+#define ARROW_JSON
+
 /* #undef ARROW_S3 */
 #define ARROW_USE_NATIVE_INT128
 
